# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base
  include UserInfo
  # Esto es para poder usar el number_with_delimiter
  #include ActionView::Helpers::NumberHelper

  helper :layout, :nueva_edicion
  before_filter :configurar_idioma
  before_filter :recuperar_sesion_callback, :only => [:modificar_crear_callback]
  before_filter :autorizar, :sesion_timeout, :objeto_seccion, :xls_request, :tiempo_comienzo, :por_pagina, :autorizar_rol, :except => [:entrada, :identificacion]

  # helper :all # include all helpers, all the time
  #protect_from_forgery # See ActionController::RequestForgeryProtection for details
  layout "layout", :except => ["entrada"] #Siempre incluimos el mismo layout general menos en identificacion

  # Comprobacion de inactividad. Configurado a 30 minutos.
  def sesion_timeout
    session[:timeout] ||= Time.now
    if Time.now - session[:timeout] > 3600
      session[:usuario_identificado_id] = nil
      session[:timeout] = Time.now
      mensaje = _("Tiempo de inactividad en el sistema superado. Vuelva a identificarse para acceder.")
      if request.xhr?
        # Si la peticion es ajax
        render (:update) {|page|  page.mensaje_informacion params[:update], mensaje, :tipo_mensaje => "mensajefallo"} if params[:update]
        render (:update) {|page|  page.mensaje_informacion params[:update_listado], mensaje, :tipo_mensaje => "mensajefallo"} if params[:update_listado]
      else
        # Si la peticion no es ajax
        flash[:mensaje] = mensaje
        session[:ir_a] = "#{request.fullpath}"
        redirect_to entrada_path
      end
    end
    session[:timeout] = Time.now
  end

  # Cada peticion se mira el idioma de la session del usuario y se configura el locale.
  def configurar_idioma
    if session[:idioma_seleccionado].nil? or session[:idioma_seleccionado] == ""
      #set_locale "es_ES"
      FastGettext.locale = I18n.locale = ENV['LANG']||'es_ES'
    else
      #set_locale session[:idioma_seleccionado]
      FastGettext.locale = I18n.locale = session[:idioma_seleccionado] 
    end
  end 

  def por_pagina
   session[:por_pagina] = params[:por_pagina] if params[:por_pagina]
  end

  # Metodo general para representar al final del layout cuando se esta en modo development el tiempo total de una peticion.
  def tiempo_comienzo     
    @tiempo_comienzo = Time.now
  end

  # Scrub sensitive parameters from your log
  # filter_parameter_logging :password

  # Este metodo general esta aqui para que los controladores puedan crear objetos fechas desde parametros del date_select
  # FIXME: Parece algo tonto pero no consigo resolverlo de otra forma
  def fecha parametros
     return Date.new parametros["(1i)"].to_i, parametros["(2i)"].to_i, parametros["(3i)"].to_i
  end

  # Se comprueba si el acceso tiene un usuario identificado, y se comprueba tambien que el usuario tiene permisos para acceder la seccion en la que se esta navegando.
  def autorizar
    #puts "------------------> ERROR!!!!! No existe la variable de sesion para usuario_identificado_id" unless session[:usuario_identificado_id]
    #puts "------------------> Tenemos la sesion: " + session.inspect
    @usuario_identificado = Usuario.find_by_id session[:usuario_identificado_id]
    #puts "------------------> @usuario_identificado -> " + @usuario_identificado.inspect
    unless @usuario_identificado.class.name == "Usuario" 
      msg_error _("Debe identificarse para acceder al sistema.")
      session[:ir_a] = "#{request.fullpath}"
      redirect_to entrada_path
      return false
    end
    
    unless params[:seccion] == "inicio" or params[:seccion] == "info" or ( @usuario_identificado and @usuario_identificado.send( params[:seccion] ) == true )
      msg_error _( "no tiene los derechos para ir a la sección " ) + params[:seccion]
      redirect_to :seccion => :inicio, :controller => :tarea
      return false
    end

    UserInfo.current_user = @usuario_identificado 
  end

  def autorizar_rol
    # Si no está autorizado, devuelve error
    if @proyecto || @agente
      autorizado = autorizar_rol_secciones
      if !autorizado
        if request.xhr?
          render :update do |page|
            page.mensaje_cambio "mensaje_cabecera_2", :errors => _("No tiene permisos suficientes para realizar esta acción.")
            page.replace_html "MB_content", :partial => "comunes/derecho_insuficiente" 
            page.call("Modalbox.resizeToContent")
          end
          #render :partial => "comunes/derecho_insuficiente"
        else
          render "comunes/derecho_insuficiente"
        end
      end
      return autorizado
    end
  end

  def autorizar_rol_secciones
    # Por defecto, denegamos
    autorizado = false
    # Para proyectos, empezamos a filtrar...
    if @proyecto
      # Si el usuario tiene rol...
      if rol = @proyecto.usuario_autorizado?(@usuario_identificado)
        autorizado = true
        # Para acciones de "modificar_crear_", "anadir_", "eliminar_", etc..
        if params[:action] =~ /^(anadir|nuevo|editar|modificar|crear|eliminar)/
          # En algunos controladores, solo puede modificar el coordinador
          if params[:controller] == "estado" || params[:controller] == "datos_proyecto" || params[:controller] == "tasa_cambio" || params[:controller] == "subpartida" || params[:controller] == "partida_financiacion"
            if rol.nil? || ( rol != "coordinador" && rol != "configurador")
              autorizado = false  
            end
          # En el resto solo coordinador y usuarios
          else
            if rol.nil? || ( rol != "coordinador" && rol != "usuario")
              autorizado = false
            end
          end
        end
        # Para el controlador de importacion, depende del menu donde estemos
        if params[:controller] == "importacion"
          if params[:menu] == "configuracion"
            autorizado = false if rol.nil? || ( rol != "coordinador" && rol != "configurador")
          else
            autorizado = false if rol.nil? || ( rol != "coordinador" && rol != "usuario")
          end
        end
      # Si el usuario no tiene rol, no autorizamos
      end
    end
    # Y para agentes...
    if @agente
      # Obtenemos el rol
      rol = @agente.usuario_autorizado?(@usuario_identificado)
      # Para algunos, solo puede entrar el coordinador 
      if params[:controller] == "proyecto"
        autorizado = rol == "coordinador"
      # Para otros, solo el economico o el administrador
      elsif params[:controller] =~ /^(estado|libro|datos_agente|presupuesto_agentes|subpartida|gasto_agente|transferencia|resumen_agente|exportacion|importacion)/ 
        autorizado = ( rol == "coordinador" || rol == "economico" )
      else
        autorizado = rol
      end
      #puts "---------> Rol: " + rol.to_s if rol
      #puts "---------> SIN rol" unless rol
      #puts "---------> El usuario SI esta autorizado en el agente para el controlador " + params[:controller].to_s if autorizado
      #puts "---------> El usuario NO esta autorizado en el agente para el controlador " + params[:controller].to_s unless autorizado
    end
    return autorizado
  end

  def singularizar_seccion
    params[:seccion].gsub(/(.*)(s)/) {$1.gsub(/one/, "on")}
  end
 
  # Este metodo devuelve la entidad dentro de la seccion con la que estamos trabajando. 
  # En el caso de la seccion proyectos pone en @proyecto el proyecto con el que estamos trabajando
  # En el caso de la seccion agentes pone en @agente el agente con el que estamos trabajando
  def objeto_seccion
    case params[:seccion]
    when "proyectos" then
      # Hacemos este apaño por causa de la subida de imagenes en tiny_mce (donde viaja el proyecto_id en params[:hint])
      params[:proyecto_id] ||= params[:hint] if params[:hint]
      @proyecto = Proyecto.find_by_id(params[:proyecto_id]) if params[:proyecto_id]
      @permitir_configuracion = @proyecto && ( @proyecto.definicion_estado.nil? || !@proyecto.definicion_estado.cerrado ) && params[:menu] == "configuracion"
      @permitir_formulacion = @proyecto && @proyecto.definicion_estado && !@proyecto.definicion_estado.cerrado && @proyecto.definicion_estado.formulacion && params[:menu] == "formulacion"
      @permitir_ejecucion = @proyecto && @proyecto.definicion_estado && !@proyecto.definicion_estado.cerrado && @proyecto.definicion_estado.aprobado && (params[:menu] == "ejecucion_tecnica" || params[:menu] == "ejecucion_economica")
    when "agentes"
      then @agente = Agente.find(params[:agente_id]) if params[:agente_id] 
    end
  end

  # Cambia el formato de respuesta cuando se pide un XLS
  def xls_request
    request.format = :xls if (params[:selector] && (params[:selector][:salida] == 'fichero' || params[:selector][:fichero] == '1')) || params[:fichero] == "1"
    request.format = :docx if params[:selector] && params[:selector][:docx] == '1'
  end

  # Para usar en exportaciones a XLS
  def xls_filename
    params[:seccion] + "-" + params[:controller] + "-" + params[:action] + ".xls"
  end



  #--
  # MENSAJES
  #++

  def msg objeto, otros={}
    cadena = nil
    if objeto.class == String
      cadena = objeto.to_s
    elsif objeto.class == Array
      cadena = "<ul>" + objeto.inject("") {|suma, e | suma + "<li>" + e + "</li>"} + "</ul>"
    elsif objeto.methods.include?(:errors)
      if objeto.errors.empty?
        otros[:ok] = true
        cadena = _("Los datos se han guardado correctamente.") unless otros[:eliminar]
        cadena = _("Se ha eliminado correctamente.") if otros[:eliminar]
      else
        otros[:error] = true
        cadena = _("Se han producido errores.") + "<br>"
        objeto.errors.each {|a, m| cadena << m + "<br>" }
      end 
    end
    if cadena
      if otros[:now]
        flash.now[:mensaje_error] = cadena if otros[:error]
        flash.now[:mensaje_ok] = cadena if otros[:ok]
        flash.now[:mensaje] = cadena unless otros[:error] || otros[:ok]
      else
        flash[:mensaje_error] = cadena if otros[:error]
        flash[:mensaje_ok] = cadena if otros[:ok]
        flash[:mensaje] = cadena unless otros[:error] || otros[:ok]
      end
    end
  end

  # dibuja el mensaje de error o de exito
  def msg_error mensaje, otros={}
    otros[:error] = true 
    msg mensaje, otros 
  end

  # dibuja el mensaje de error o de exito al eliminar un objeto
  def msg_eliminar objeto
    msg objeto, :eliminar => true
  end


  # Conversiones entre formatos de moneda accesibles desde todos los controladores
  def float_a_moneda numero 
    #number_with_delimiter(('%.2f' % numero).to_s , :separator => ",", :delimiter => ".") if numero
    ('%.2f' % numero).to_s.sub(".",",") if numero
  end

  def moneda_a_float cadena
    if cadena
      # primero comprobamos que el punto corresponde a miles ( formato => n.nnn )
      # y solo en ese caso lo eliminamos => Permitimos especificar decimales con punto
      numero = (cadena =~ /[\d]+\.[\d]{3}/) ? cadena.delete(".") : cadena
      # Si se esta usando la coma decimal, la cambiamos por un punto ( formato => n,nn o n,n )
      # REVISAR: Esto permite que haya coma para miles, pero impide que metan mas de 2 decimales con coma
      numero = numero.sub(/,/,".") if numero =~ /[\d]+,[\d]{1,2}$/
      return numero.to_f
    end
  end

end
