# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de las vistas resumen financieras para una agente implementador. Este controlador es utilizado desde las secciones agentes.

class ResumenAgenteController < ApplicationController

  helper :resumen_proyecto

  before_filter :verificar_etapa

  def verificar_etapa
    if @agente.nil? || @agente.etapa.empty?
      msg_error _("Tiene que definir por lo menos una etapa para acceder a los informes")
      redirect_to :menu => :configuracion_agente, :controller => :datos_agente, :action => :etapas
    end
  end

  
  # en agente: se redirecciona por defecto a presupuesto
  def index
    redirect_to :action => :presupuesto
  end

  def transferencia
    @listado_moneda = @agente.moneda.collect{|e| [e.nombre, e.id.to_s]}
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}

    # Hacemos la busqueda si se seleccionan los criterios
    estado_transferencia if params[:moneda]

    respond_to do |format|
      format.xls do
        nom_fich = "transferencias_proyectos_" + @agente.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
      end
      format.html
    end
  end

  def estado_transferencia
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = fecha params["fecha_fin"]
    if params[:seleccion_fecha] == "etapa" && params[:etapa] && etapa = Etapa.find_by_id(params[:etapa])
      @fecha_de_inicio = etapa.fecha_inicio
      @fecha_de_fin = etapa.fecha_fin
    end
    @moneda = Moneda.find_by_id params[:moneda]

    aplica_tasa_cambio = @moneda.nil? || params[:tasa_cambio] == "1"
    abreviatura_moneda = aplica_tasa_cambio ? (@proyecto || @agente).moneda_principal.abreviatura : @moneda.abreviatura

    condiciones = {"fecha_enviado" => @fecha_de_inicio..@fecha_de_fin}
    condiciones["libro.agente_id"] = @agente.id
    condiciones["libro.moneda_id"] = @moneda.id if @moneda
    transferencias=Transferencia.joins(:libro_origen,:libro_destino).where(condiciones).where("!(libro.agente_id = ? AND NOT proyecto_id)",@agente.id).group(["proyecto_id","libro_destinos_transferencia.agente_id","libro.moneda_id"]).sum(:importe_enviado)

    # Prepara la salida de datos
    lineas = Array.new
    lineas.push( :cabecera => [ [_("Proyecto"),"1"], [_("Implementador"),"1"], [_("Transferido"),"1_2_td"], ["","1_3"], [_("Gastado"),"1_2_td"], ["","1_3"] ])
    i = 0
    transferencias.each do |key,value|
      moneda = Moneda.find_by_id key[2]
      agente = Agente.find_by_id key[1]
      proyecto = Proyecto.find_by_id key[0]
      if proyecto
        nombre_proyecto = proyecto.nombre
        moneda_proyecto = aplica_tasa_cambio ? proyecto.moneda_principal.abreviatura : moneda.abreviatura
        condiciones_gasto = { "gasto_x_proyecto.proyecto_id" => proyecto.id, "gasto.agente_id" => agente.id }
        condiciones_gasto["gasto.moneda_id"] = @moneda.id if @moneda 
        gastado = GastoXProyecto.joins(:gasto).joins(:tasa_cambio_proyecto).where(condiciones_gasto).sum("gasto_x_proyecto.importe * tasa_cambio.tasa_cambio") if aplica_tasa_cambio
        gastado = GastoXProyecto.joins(:gasto).joins(:tasa_cambio_proyecto).where(condiciones_gasto).sum("gasto_x_proyecto.importe") unless aplica_tasa_cambio
      else
        nombre_proyecto = _("Sin Proyecto")
        moneda_proyecto = "" 
        gastado = "" 
      end
      lineas.push( :contenido => [ nombre_proyecto, agente.nombre, value, moneda.abreviatura, gastado, moneda_proyecto ] )
    end

    # Preparamos la tabla para mostrar el detalle del seguimiento por subpartidas del agente
    titulo_datos =[ _("Moneda") + ": " + (@moneda ? @moneda.abreviatura :  _('Todas las monedas')) ,
               _("Tasa cambio") + ": " + (aplica_tasa_cambio ? _('Aplicada'): _('No aplicada')),
               _("Fechas") + ": " + @fecha_de_inicio.to_time.to_s + ' - ' + @fecha_de_fin.to_time.to_s ]

    titulo = [_("Resumen de Transferencias a Proyectos.")] + titulo_datos

    # Recogemos todo en una tabla común
    @resumen = [ :listado => { :nombre => _("Resumen de transferencias a proyectos"), :titulo => titulo, :lineas => lineas } ]
  end

  def gasto
    @listado_moneda = @agente.moneda.collect{|e| [e.nombre, e.id.to_s]} + [["Todas las monedas (con tasa aplicada)","todas"]]
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}
    # Hacemos la busqueda si se seleccionan los criterios
    estado_gasto if params[:moneda]
  end

  # Obtiene un resumen de gastos
  # Esto hay que simplificarlo... (invocamos las mismas vistas varias veces)
  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = fecha params["fecha_fin"] 
    if params[:seleccion_fecha] == "etapa" && params[:etapa] && etapa = Etapa.find_by_id(params[:etapa])
      @fecha_de_inicio = etapa.fecha_inicio
      @fecha_de_fin = etapa.fecha_fin 
    end

    # Recogemos la tabla de resumen general
    condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
    partidas = Partida.find(:all, :conditions => condiciones, :order => "codigo")
    filas = partidas.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    columnas = [{"id" => "1", "nombre" => _("Presupuesto")}, {"id" => "2", "nombre" => _("Gastos")}]
    # buscamos el presupuesto por partidas para las fechas y criterios seleccionadas
    presupuestos = VPresupuestoAgenteDetallado.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin , params[:moneda],  params[:tasa_cambio], nil)
    presupuestos = presupuestos.collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    # Buscamos el gasto por partidas del agente para las fechas y criterios seleccionados
    gastos = VGastoAgente.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda], params[:tasa_cambio], nil, false)
    gastos = gastos.collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    # preparamos la tabla para su presentación visual
    #datos = presupuestos + gastos
    titulo_datos =[ _("Moneda") + ": #{ params[:moneda] == 'todas' ?  _('Todas las monedas') : Moneda.find(params[:moneda]).abreviatura} " ,
               _("Tasa cambio") + ": " + ((params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? _('Aplicada'): _('No aplicada')),
               _("Fechas") + ": " + @fecha_de_inicio.to_time.to_s + ' - ' + @fecha_de_fin.to_time.to_s ]

    #titulo_resumen =  [_("Resumen de gasto comparado con el presupuesto.")] + titulo_datos
    #tabla_resumen = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true, :columna_desviacion => true } }

     # Comprobamos que no hay partidas ocultas con datos de agente para comunicarlo al usuario
     # Simplificar esto usando el bucle de mas abajo (duplica los select a bbdd)
     presupuestos.each do |p|
       msg_error(_("Existen partidas ocultas que contienen presupuestos.") + " " + _("Seleccione mostrar partidas ocultas para verlas"), :now => true)  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end
     gastos.each do |p|
       msg_error(_("Existen partidas ocultas que contienen gastos.") + " " + _("Seleccione mostrar partidas ocultas para verlas"), :now => true)  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end

     # Preparamos la tabla para mostrar el detalle del seguimiento por subpartidas del agente
     @titulo_partida = [_("Resumen de gasto comparado con el presupuesto.")] + titulo_datos
     @titulo_subpartida = [_("Detalle del presupuesto frente al gasto del agente por subpartidas y el gasto imputado a delegacion frente al gasto imputado a proyectos")] + titulo_datos
     @partidas = [:cabecera => [[_("Partida"), "1"], [_("Presupuesto"),"1_2_td"], [ _("Gasto") ,"1_2_td"], [_("Diferencia") ,"1_2_td"],  [_("Desviación") + " %" ,"1_2_td"],  [_("Imputado a delegación") ,"1_2_td"],  [_("Imputado a proyecto") ,"1_2_td"]] ]
     @subpartidas =  [:cabecera => [[_("Partida/Subpartida") , "1" ], [_("Presupuesto"),"1_2_td"], [ _("Gasto") ,"1_2_td"], [_("Diferencia") ,"1_2_td"],  [_("Desviación") + " %" ,"1_2_td"],  [_("Imputado a delegación") ,"1_2_td"],  [_("Imputado a proyecto") ,"1_2_td"]] ]
     # Preparamos la tabla para mostrar el detalle de seguimiento de sbupartidas por proyectos
     @titulo_subpartidas_proyectos = [_("Detalle del gasto imputado a proyectos mostrado por subpartida y proyecto")] + titulo_datos
     @subpartidas_proyectos=  [:cabecera => [[ _("Partida/Subpartida") , "2" ], [_("Proyecto") ,"3_2"], [ _("Gasto imputado proyectos")  ,"1_2_td"],  [_("Gasto total partida")  ,"1_2_td"]]]
      # Totales
      tp = tg = tid = tip = 0
      # Recorre todas las partidas
      for partida in partidas
        # Buscamos para cada partida el presupuesto para las fechas y los criterios seleccionados
        importe_presupuesto = VPresupuestoAgenteDetallado.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s ).first
        # Buscamos para cada partida el gasto total para las fechas y los criterios seleccionados
        importe_gasto = VGastoAgente.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, false ).first
        # Buscamos para cada partida el gasto imputado a delegacion para las fechas y los criterios seleccionados
        importe_gasto_delegacion = VGastoAgente.partida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, true  ).first
        # Buscamos para cada partida el gasto imputado a proyectos para las fechas y los criterios seleccionados
        importe_gasto_proyectos = VGastoAgente.partida_asignado_proyecto(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s).first

        gasto = (importe_gasto and importe_gasto[:importe]) ? importe_gasto[:importe] : 0
        gasto_delegacion = (importe_gasto_delegacion and importe_gasto_delegacion[:importe]) ? importe_gasto_delegacion[:importe] : 0
        gasto_proyectos = (importe_gasto_proyectos and importe_gasto_proyectos[:importe]) ? importe_gasto_proyectos[:importe] : 0
        presupuesto = (importe_presupuesto and importe_presupuesto[:importe]) ? importe_presupuesto[:importe] : 0

        # Mete los datos en las tablas
        @partidas.push(:contenido => [partida.codigo_nombre, presupuesto, gasto, (presupuesto - gasto), (presupuesto == 0 ? "-" : (presupuesto - gasto)/presupuesto * 100),
			gasto_delegacion, gasto_proyectos] )
        @subpartidas.push(:cabecera => [ [ partida.codigo_nombre , "1" ], [presupuesto ,"1_2_td"], [ gasto  ,"1_2_td"], [(presupuesto - gasto)  ,"1_2_td"],
                      [ (presupuesto == 0 ? "-" : (presupuesto - gasto)/presupuesto * 100)  ,"1_2_td"], [gasto_delegacion  ,"1_2_td"], [gasto_proyectos  ,"1_2_td"]] ) 
        @subpartidas_proyectos.push(:cabecera => [[ partida.codigo_nombre , "2" ], ["" ,"3_2"], [ gasto_proyectos  ,"1_2_td"], [ gasto,"1_2_td"]])

        # Suma los totales
        tp += presupuesto
        tg += gasto
        tid += gasto_delegacion
        tip += gasto_proyectos

        # Hace los calculos para las subpartidas
        for subpartida in   Subpartida.find_all_by_agente_id_and_partida_id( @agente.id, partida.id)
          pres = VPresupuestoAgenteDetallado.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin ,params[:moneda], params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, nil)
          importe_gasto = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, nil, false)
          importe_gasto_delegacion = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, nil, true)
          importe_gasto_proyectos = VGastoAgente.subpartida_asignado_proyecto(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, false)
          gasto = importe_gasto.empty? ? 0 : importe_gasto.first[:importe]
          gasto_delegacion = importe_gasto_delegacion.empty? ? 0 : importe_gasto_delegacion.first[:importe]
          gasto_proyectos = importe_gasto_proyectos.empty? ? 0 : importe_gasto_proyectos.first[:importe]
          presupuesto = pres.empty? ? 0 : pres.first[:importe]
          @subpartidas.push(:contenido => [subpartida.nombre, presupuesto, gasto, (presupuesto - gasto), (presupuesto == 0 ? "-" : (presupuesto - gasto)/presupuesto * 100), gasto_delegacion, gasto_proyectos])  
          # Ahora ponemos los datos para rellenar la tabla de proyectos por partidas
          proyecto_subpartida = VGastoAgente.subpartida_por_proyecto(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, subpartida.id.to_s, false, true)
          for fila in proyecto_subpartida
            @subpartidas_proyectos.push(:contenido => [(fila.subpartida_nombre || _("Sin subpartida")), (fila.proyecto_nombre|| _("Imputado a delegación")), (fila.importe || "0")])
          end

        end
        importe_pre = VPresupuestoAgenteDetallado.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin ,params[:moneda], params[:tasa_cambio], partida.id.to_s, nil, true)
        importe_gasto = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, nil, true, false)
        importe_gasto_delegacion = VGastoAgente.subpartida(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, nil, true, true)
        importe_gasto_proyectos = VGastoAgente.subpartida_asignado_proyecto(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, nil, true)
        gasto = (importe_gasto.empty? or importe_gasto.first[:importe].nil?) ? 0 : importe_gasto.first[:importe]
        gasto_delegacion = (importe_gasto_delegacion.empty? or importe_gasto_delegacion.first[:importe].nil?) ? 0 : importe_gasto_delegacion.first[:importe]
        gasto_proyectos = (importe_gasto_proyectos.empty? or importe_gasto_proyectos.first[:importe].nil?) ? 0 : importe_gasto_proyectos.first[:importe]
        pres = (importe_pre.empty? or importe_pre.first[:importe].nil?) ? 0 : importe_pre.first[:importe]
        @subpartidas.push(:contenido => [_("Sin subpartida"), pres, gasto, (pres - gasto), (pres == 0 ? "-" : (pres - gasto)/pres * 100), gasto_delegacion, gasto_proyectos])  unless gasto == 0 and pres == 0
        proyecto_subpartida = VGastoAgente.subpartida_por_proyecto(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda] , params[:tasa_cambio], partida.id.to_s, nil, true, true)
        for fila in proyecto_subpartida
          @subpartidas_proyectos.push(:contenido => [(fila.subpartida_nombre || "Sin subpartida"), (fila.proyecto_nombre || ""), (fila.importe || "0")])
        end
    end

    # Mete los totales por columna para las partidas
    @partidas.push(:contenido => [_("TOTALES"), tp, tg, (tp - tg), (tp == 0 ? "-" : (tp - tg)/tp * 100), tid, tip])

    #@tablas =[tabla_resumen]

    # Recogemos todo en una tabla común
    respond_to do |format|
      format.xls do
        #@resumen = [ :tabla => tabla_resumen ]
        @resumen = []
        @resumen.push(  :listado => {   :nombre => _("Resumen por partidas"),
                                        :titulo => @titulo_partida,
                                        :lineas => @partidas })
        @resumen.push(	:listado => {	:nombre => _("Resumen por subpartidas"),
					:titulo => @titulo_subpartida,
					:lineas => @subpartidas }) 
        @resumen.push(	:listado => {	:nombre => _("Resumen por subpartidas y proyectos"),
					:titulo => @titulo_subpartidas_proyectos,
					:lineas => @subpartidas_proyectos }) 
        nom_fich = "resumen_gasto_" + @agente.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
      end
      format.html
    end
  end

  def presupuesto
    @listado_moneda = (@agente.moneda.collect{|e| [e.nombre, e.id]}) + [["Todas las monedas (T.Cambio aplicada)", "todas"]]
    @listado_etapas = @agente.etapa.collect{|e| [e.nombre, e.id]}
    estado_presupuesto if params[:moneda] 
  end

  def estado_presupuesto
   condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
   partidas = Partida.find(:all, :conditions => condiciones, :order => "codigo")
   # Recogemos los datos de la tabla primera resumen del presupuesto de agente frente al presupuesto de proyecto
   filas =  partidas.collect {|i| {"id" => i.id, "nombre" => i.codigo_nombre} }  # if params[:detallado] == "1"       
   columnas = [{"id" => "2", "nombre" => "Pto. Proyectos"}, {"id" => "1", "nombre" => "Pto. Agente"}]
   etapa = Etapa.find params[:etapa]
   presupuestos_proyectos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], nil, true).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}
   presupuestos_delegacion = VPresupuestoAgente.partida_implementador(@agente.id, etapa.id, params[:moneda], params[:tasa_cambio], nil).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
   datos = presupuestos_proyectos + presupuestos_delegacion
   titulo =[ "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
             "Tasa cambio: #{(params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
             "Etapa: #{ etapa.nombre + ' (' + etapa.fecha_inicio.to_time.to_s + ' - ' + etapa.fecha_fin.to_time.to_s + ")" } " ]
   titulo_resumen = ["Resumen del presupuesto de proyectos (ingresos) y de agente (gastos)"] + titulo
   tabla = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true } }

   # Comprobamos que no hay partidas ocultas con datos de agente
   presupuestos_proyectos.each do |p|
     msg_error(_("Existen partidas ocultas que contienen presupuestos de proyecto.") + " " + _("Seleccione mostrar partidas ocultas para verlas"), :now => true)  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
   end
   presupuestos_delegacion.each do |p|
     msg_error(_("Existen partidas ocultas que contienen presupuestos de agente.") + " " + _("Seleccione mostrar partidas ocultas para verlas"), :now => true)  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
   end

   #Recogemos la tabla del detalle del presupuesto del agente por subpartidas y la de detalle de presupuesto por proyectos   
   @subpartidas = Array.new
   @presupuesto_proyectos = Array.new
   for partida in partidas
      #Buscamos el valor por partidas del presupuesto por agente
      importe_partida = VPresupuestoAgente.partida_implementador(@agente.id, etapa.id, params[:moneda], params[:tasa_cambio], partida.id).first
      @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [(importe_partida ? importe_partida[:importe] : 0) ,"1_2_td"]])
    
      for subpartida in  Subpartida.find_all_by_agente_id_and_partida_id( @agente.id, partida.id)
        valor = VPresupuestoAgente.subpartida_partida(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio], partida.id.to_s, (subpartida ? subpartida.id.to_s : nil))
        @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "sin subpartida"), valor.first[:importe]]) unless valor.nil? or valor.empty?
      end
      #buscamos el valor por partidas del presupuesto por proyecto
      partida_proyectos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], partida.id, true).first
      @presupuesto_proyectos.push(:cabecera => [[ partida.codigo_nombre , "2" ], ["","2"], [(partida_proyectos  ? partida_proyectos[:importe] : 0) ,"1_2_td"]]) 
      detalle_partida_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], partida.id, true)
      for detalle in detalle_partida_proyecto
         @presupuesto_proyectos.push(:contenido => [detalle.proyecto_nombre,  detalle.presupuesto_concepto,(detalle.importe || 0) ]) 
      end
    end
     @titulo_presupuesto_subpartida = ["Detalle del presupuesto del agente (gastos) por subpartidas"] + titulo   
     @titulo_presupuesto_proyectos = ["Detalle del presupuesto por proyectos (ingresos)"] + titulo
     # Recogemos todo en una tabla común
     @tablas = [tabla]
     respond_to do |format|
        format.xls do
          @nombre = "Resumen Presupuesto" + @agente.nombre
          @resumen = [  :tabla => tabla ]
          @resumen.push( :listado => {  :tipo => "v_presupuesto_subpartida",
                                        :nombre => _("Resumen por subpartidas"),
                                        :titulo => @titulo_presupuesto_subpartida,
                                        :lineas => @subpartidas })
          @resumen.push( :listado => {	:tipo => "v_presupuesto_proyecto",
					:nombre => _("Resumen por proyectos"),
					:titulo => @titulo_presupuesto_proyectos,
					:lineas => @presupuesto_proyectos })

          nom_fich = "resumen_presupuesto_" + @agente.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
        format.html
      end
  end

  # Ya no se utiliza!
  def subpartida
   @columnas_agente = [{"id" => "1", "nombre" => "Importe"}]
   @filas_agente = Presupuesto.find(:all, :select => "subpartida.nombre", :include => [:subpartida], :conditions => {"agente_id" => @agente.id, "partida_id" => params[:partida], :proyecto_id => nil} , :group => "subpartida.nombre" ).collect{ |p| ({"nombre" => p.subpartida.nombre, "id" =>  p.subpartida.id }) if p.subpartida }
   @etapa = Etapa.find(params[:etapa])
   @datos_agente = VPresupuestoAgente.subpartidas_partida(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio], params[:partida]).collect{|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}

   @columnas_proyecto = [{"id" => "1", "nombre" => "Importe"} ]
   presupuesto_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, @etapa.fecha_inicio, @etapa.fecha_fin , params[:tasa_cambio], params[:partida])
   @filas_proyecto = presupuesto_proyecto.collect {|p| {"nombre" => p.proyecto_nombre , "id" => p.presupuesto_id}}
   @datos_proyecto  = presupuesto_proyecto.collect {|p| {"columna_id" => "1" , "fila_id" => p.presupuesto_id.to_s, "importe" => p.importe}}
  
   render :partial => "subpartida" 
  end

	# en agentes: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| @usuario_identificado.libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}

    # Si hemos especificado un libro para ver el informe
    if params[:selector] and params[:selector][:libro] != ''
      @libro = Libro.find(params[:selector][:libro])
      @moneda = @libro.moneda
      @etapa = Etapa.find_by_id(params[:selector][:etapa])

      fecha_inicio = @etapa ? @etapa.fecha_inicio : nil
      fecha_fin = @etapa ? @etapa.fecha_fin : nil

      arqueo = @libro.arqueo [], fecha_inicio, fecha_fin

      # Incluye los saldos linea a linea
      saldo = 0.0
      arqueo[:filas].each do |fila|
        saldo += fila[:contenido][3].to_f - fila[:contenido][4].to_f
        fila[:contenido].push saldo
      end

      # Prepara la salida de datos
      lineas = Array.new
      lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"], [_("Saldo"),"1_2_td"] ])
      lineas += arqueo[:filas]
      lineas.push( :contenido => [ '','','','','','' ] )
      lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [arqueo[:entrante],"1_2_td"], [arqueo[:saliente],"1_2_td"], [arqueo[:entrante] - arqueo[:saliente],"1_2_td"] ])
      arqueo[:totales].each do |k,v|
        lineas.push( :contenido => [ '',_(k),'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      @resumen = Array.new
      nombre = "arqueo_caja"
      titulo = _("Arqueo Cuenta/Caja") + " " + @libro.nombre + " " + @libro.cuenta + " (" + @libro.moneda.nombre + ") / " + _("Etapa") + ": " +
		( @etapa ? @etapa.nombre + " (" + _("desde") + " " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " " + _("hasta") + " " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : _("Todas"))
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

      respond_to do |format|
        format.html do
          render :template => 'comunes/arqueo_caja', :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          nom_fich = "arqueo_" + @agente.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end
    else
      render :template => "comunes/arqueo_caja", :layout => (params[:sin_layout] ? false : true)
    end
 
  end

end
