# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de los resumenes de proyecto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para presentar un resumen de proyecto según diversos parámetros

class ResumenProyectoController < ApplicationController

  before_filter :verificar_etapa
  before_filter :criterios_gasto, :only => [:gasto]
  before_filter :criterios_presupuesto, :only => [:presupuesto]

  def verificar_etapa
    if @proyecto.etapa.empty?
      msg_error _("Tiene que definir por lo menos una etapa para acceder a los informes")
      redirect_to :menu => :configuracion, :controller => :datos_proyecto, :action => :etapas
    end
  end

  def criterios_gasto
    if params[:financiador_implementador] == "" 
      msg_error _("Debe seleccionar agente. Para ello primero seleccione tipo implementador o financiador.")
      redirect_to :action => "gasto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end

  def criterios_presupuesto
    if params[:selector] and (params[:selector][:etapa].nil? or params[:selector][:etapa]=="" )
      msg_error _("Debe seleccionar etapa. Si no existe ninguna debe dar de alta para este proyecto.")
      redirect_to :action => "presupuesto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end



  def index
    redirect_to :action => 'presupuesto'
  end
  
        # en proyectos: selecciona la moneda y llama a estado_presupuesto si ya hay moneda y actividad/partida
  def presupuesto
    @listado_moneda = [[_("Todas (con tasa aplicada)"), "todas"]] + @proyecto.moneda.collect{|e| [e.nombre, e.id]}
    @listado_etapa =[[_("Todas"), "todas"]] +  @proyecto.etapa.collect{|e| [e.nombre, e.id]}

    unless params[:selector]
      params[:selector] = {:tasa_cambio => "0", :fichero => "0"}
      params[:selector][:moneda] = "todas"
      params[:selector][:partida_actividad] = "partida_proyecto"
      params[:selector][:financiador_implementador] = "financiador"
      params[:selector][:etapa] = @proyecto.etapa.last.id.to_s if @proyecto.etapa.last
    end

    # Hacemos la busqueda si se seleccionan los criterios
    if params[:selector] and params[:selector][:moneda] != '' and params[:selector][:partida_actividad] != '' and params[:selector][:etapa] != '' and params[:selector][:etapa]
      estado_presupuesto 
      respond_to do |format|
        format.html do 
          render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          @nombre = _("Resumen Presupuesto")
          #render 'comunes_xls/resumen_tabla', :layout => false
          nom_fich = "resumen_presupuesto_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end
    else
      render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
    end
  end

  def estado_presupuesto
    partidas = Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    actividades = @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    implementadores = @proyecto.implementador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    financiadores = @proyecto.financiador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    partidas_proyecto = @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    
    case  [params[:selector][:partida_actividad],  params[:selector][:financiador_implementador]]
      when ["partida", "implementador"] then
         datos = VPresupuesto.partida_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, implementadores
      when ["partida", "financiador"] then
         datos = VPresupuesto.partida_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, financiadores
      when ["actividad", "implementador"] then
         datos = VPresupuesto.actividad_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = actividades, implementadores
      when ["actividad", "financiador"] then
         datos = VPresupuesto.actividad_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = actividades, financiadores
      when ["partida_proyecto", "implementador"] then
         datos = VPresupuesto.partida_proyecto_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, implementadores
      when ["partida_proyecto", "financiador"] then
         datos = VPresupuesto.partida_proyecto_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, financiadores
      when ["actividad", "financiadores_agrupados"] then
         filas = actividades
         columnas,datos = VPresupuesto.actividad_financiador_x_grupos @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
      when ["partida", "financiadores_agrupados"] then
         filas = partidas
         columnas,datos = VPresupuesto.partida_financiador_x_grupos @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
      when ["partida_proyecto", "financiadores_agrupados"] then
         filas = partidas_proyecto
         columnas,datos = VPresupuesto.partida_proyecto_financiador_x_grupos @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
    end       

    nom_fila = case params[:selector][:partida_actividad]
      when "partida_proyecto"
        _("Partidas del proyecto")
      when "partida"
        _("Partidas")
      when "actividad"
        _("Actividades")
    end
    titulo =[ _("Resumen de presupuesto."),
              _("Moneda") + ": " + (params[:selector][:moneda] == 'todas' ? _('Todas las monedas') : Moneda.find(params[:selector][:moneda]).abreviatura) ,
              _("Tasa Cambio") + ": " + ( (params[:selector][:tasa_cambio] == '1' or params[:selector][:moneda] == 'todas') ? _('Aplicada'): _('No aplicada') ) + " (" +
                _("importes en %{moneda}") % {:moneda => Moneda.find_by_id( (params[:selector][:tasa_cambio] == '1' or params[:selector][:moneda] == 'todas') ? @proyecto.moneda_id : params[:selector][:moneda]).abreviatura } + ").",
              _("Etapa") + ": " + (params[:selector][:etapa] == 'todas' ? _('Todas las etapas') : Etapa.find(params[:selector][:etapa]).nombre),
              _("Filas") + ": " +  nom_fila,
	      _("Columnas") + ": " + _(params[:selector][:financiador_implementador].humanize.capitalize) ]
    
    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => { :columna_suma => true, :fila_suma => true, :ancho_fila => ((columnas.count + 2) * 105 + 200), :fila_porcentaje => true , :columna_porcentaje => true} } ]
    @resumen = [ :tabla => {:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => { :columna_suma => true, :fila_suma => true, :ancho_fila => ((columnas.count + 2) * 105 + 200), :fila_porcentaje => true , :columna_porcentaje => true} } ]

    nombre_subpartidas = _("Subpartidas por partida")

    @titulo, @titulo[0] = titulo[0..3], nombre_subpartidas 

    # Buscamos las subpartidas para segunda tabla si estamos en un resumen por partidas o por partidas de proyecto.
    @subpartidas = Array.new
    if params[:selector][:partida_actividad] == "partida"
      for partida in Partida.all
          importe_partida = VPresupuesto.partida(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [importe_partida ,"1_2_td"]])
        for subpartida in  Subpartida.find_all_by_proyecto_id_and_partida_id( @proyecto.id, partida.id)
          importe = VPresupuesto.subpartida_partida(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "sin subpartida"), importe]) if importe
        end
      end
    elsif  params[:selector][:partida_actividad] == "partida_proyecto"
      for partida_proyecto in @proyecto.partida_financiacion
          importe_partida = VPresupuesto.partida_proyecto(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida_proyecto.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida_proyecto.codigo_nombre , "2" ], [importe_partida ,"1_2_td"]])
        for subpartida in partida_proyecto.subpartida_proyecto @proyecto.id
          importe = VPresupuesto.subpartida_partida_proyecto(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "Sin subpartida"), importe]) if importe
        end
      end
    end

    @resumen.push( :listado => {:nombre => nombre_subpartidas, :titulo => @titulo, :lineas => @subpartidas} ) if @subpartidas
  end

  # en proyectos: selecciona la moneda, los agentes
  # y llama a busqueda_presupuesto
  def gasto
    @listado_moneda = [[_("Todas las monedas (con tasa aplicada)"), "todas"]] + @proyecto.moneda.collect{|e| [e.nombre, e.id]}
    @listado_agente = [[_("Todos los agentes"), "todos"]] + @proyecto.implementador.collect{|e| [e.nombre, e.id]}
    @listado_pac = [ [_("Acumulado (todos los PACs)"), @proyecto.id] ] + @proyecto.pacs.collect{|e| [e.nombre, e.id]} if @proyecto.convenio?
    # Hacemos la busqueda si se seleccionan los criterios

    params[:moneda] ||= "todas"
    params[:partida_actividad] ||= "partida_proyecto"
    params[:financiador_implementador] ||= "financiador"
    params[:agente] ||= @proyecto.agente.id
    params[:pac] ||= @proyecto.pacs.first.id if @proyecto.convenio?
    @fecha_de_inicio = @proyecto.etapa.first.fecha_inicio unless params["fecha_inicio"] 
    #@fecha_de_fin = DateTime.now.to_date unless params["fecha_fin"]
    @fecha_de_fin = @proyecto.fecha_de_fin unless params["fecha_fin"]
    
    unless params[:moneda].nil? or params[:partida_actividad].nil? or params[:financiador_implementador].nil? 
      estado_gasto
      respond_to do |format|
        format.html do 
          render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          @nombre = _("Seguimiento Gasto")
          nom_fich = "seguimiento_gasto_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end
    else
      render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
    end
  end

  def agentes
    render :partial => "agentes"
  end


  def estado_gasto
    @fecha_de_inicio ||= fecha params["fecha_inicio"]
    @fecha_de_fin ||= fecha params["fecha_fin"]
    #@fecha_de_fin = (@fecha_de_fin >> 1) - 1 # Para que incluya el mes en elegido
    proyecto = Proyecto.find_by_id(params[:pac]) if @proyecto.convenio?
    proyecto = @proyecto unless @proyecto.convenio?
    filas = case params[:partida_actividad]
      when "partida" then Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
      when "actividad" then proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
      when "resultado" then proyecto.resultado.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}} + [{"id" => 0, "nombre" => _("Actividades Globales")}]
      when "objetivo_especifico" then proyecto.objetivo_especifico.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}} + [{"id" => 0, "nombre" => _("Actividades Globales")}]
      when "partida_proyecto" then proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
      when "neutro" then Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    end
    columnas = [{"id" => "1", "nombre" => _("Presupuesto")}, {"id" => "2", "nombre" => _("Gastos")}] unless params[:agente] == "todos_agrupados"
    columnas = VPresupuesto.columnas_financiador_agrupado(@proyecto) if params[:agente] == "todos_agrupados"

    # Establece el filtro de partidas por defecto
    filtro_partida = nil
    # Si hemos puesto un filtro y estamos sacando un agrupado por actividad, resultado o objetivos, establecemos el filtro
    if (params[:partida_actividad] == "actividad" || params[:partida_actividad] != "resultado" || params[:partida_actividad] != "objetivo_especifico") && params[:filtro_partida]
      # En el caso de que exista la partida de financiacion 
      if partida_financiacion = PartidaFinanciacion.find_by_id( params[:filtro_partida] )
        # Si es una partida madre
        if partida_financiacion.padre
          filtro_partida = []
          partida_financiacion.partida_financiacion_hija.each{|par| filtro_partida.push(par.partida.collect{|p| p.id})}
        else
          filtro_partida = partida_financiacion.partida.collect{|p| p.id}
        end
      end
    end

    case  [params[:partida_actividad],  params[:financiador_implementador]]
      when ["neutro", "implementador"] then
         presupuestos = []
         gastos = VGasto.actividad_por_partida( proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}
         columnas = proyecto.actividad(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}

      when ["partida", "implementador"] then
         agente = params[:agente]
         presupuestos = VPresupuestoDetallado.partida_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.agrupa_partida_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])

      when ["partida", "financiador"] then
        # Si queremos el resumen agrupado...
        if params[:agente] == "todos_agrupados"
          presupuestos = []
          gastos = VGasto.partida_financiador_x_grupos( proyecto.id, params[:moneda], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])
        # Para resumen por agente...
        else
          # Para los agentes del sistema, seleccionamos la agrupacion...
          agt = Agente.find_by_id params[:agente]
          if agt && agt.sistema
            if agt.publico
              agente = agt.local ? @proyecto.financiador_local_publico : @proyecto.financiador_externo_publico
            else
              agente = agt.local ? @proyecto.financiador_local_privado : @proyecto.financiador_externo_privado
            end 
          end
          agente ||= params[:agente]
          # Y calculamos presupuesto y gasto
          presupuestos = VPresupuestoDetallado.partida_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
          gastos = VGasto.agrupa_partida_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])
        end


      when ["actividad", "implementador"] then
         agente = params[:agente]
         presupuestos = VPresupuestoDetallado.agrupa_actividad_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
         gastos = VGasto.agrupa_actividad_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)

      when ["resultado", "implementador"] then
         agente = params[:agente]
         presupuestos = VPresupuestoDetallado.agrupa_resultado_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
         gastos = VGasto.agrupa_resultado_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)

      when ["objetivo_especifico", "implementador"] then
         agente = params[:agente]
         presupuestos = VPresupuestoDetallado.agrupa_oe_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
         gastos = VGasto.agrupa_oe_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)


      when ["actividad", "financiador"] then
        # Si queremos el resumen agrupado...
        if params[:agente] == "todos_agrupados"
          presupuestos = []
          gastos = VGasto.actividad_financiador_x_grupos( proyecto.id, params[:moneda], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        # Para resumen por agente...
        else
          # Para los agentes del sistema, seleccionamos la agrupacion...
          agt = Agente.find_by_id params[:agente]
          if agt && agt.sistema
            if agt.publico  
              agente = agt.local ? @proyecto.financiador_local_publico : @proyecto.financiador_externo_publico
            else
              agente = agt.local ? @proyecto.financiador_local_privado : @proyecto.financiador_externo_privado
            end 
          end
          agente ||= params[:agente]
          # Y calculamos presupuesto y gasto
          presupuestos = VPresupuestoDetallado.agrupa_actividad_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
          gastos = VGasto.agrupa_actividad_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        end

      when ["resultado", "financiador"] then
        # Si queremos el resumen agrupado...
        if params[:agente] == "todos_agrupados"
          presupuestos = []
          gastos = VGasto.resultado_financiador_x_grupos( proyecto.id, params[:moneda], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        # Para resumen por agente...
        else
          # Para los agentes del sistema, seleccionamos la agrupacion...
          agt = Agente.find_by_id params[:agente]
          if agt && agt.sistema
            if agt.publico
              agente = agt.local ? @proyecto.financiador_local_publico : @proyecto.financiador_externo_publico
            else
              agente = agt.local ? @proyecto.financiador_local_privado : @proyecto.financiador_externo_privado
            end
          end
          agente ||= params[:agente]
          # Y calculamos presupuesto y gasto
          presupuestos = VPresupuestoDetallado.agrupa_resultado_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
          gastos = VGasto.agrupa_resultado_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        end

      when ["objetivo_especifico", "financiador"] then
        # Si queremos el resumen agrupado...
        if params[:agente] == "todos_agrupados"
          presupuestos = []
          gastos = VGasto.oe_financiador_x_grupos( proyecto.id, params[:moneda], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        # Para resumen por agente...
        else
          # Para los agentes del sistema, seleccionamos la agrupacion...
          agt = Agente.find_by_id params[:agente]
          if agt && agt.sistema
            if agt.publico
              agente = agt.local ? @proyecto.financiador_local_publico : @proyecto.financiador_externo_publico
            else
              agente = agt.local ? @proyecto.financiador_local_privado : @proyecto.financiador_externo_privado
            end
          end
          agente ||= params[:agente]
          # Y calculamos presupuesto y gasto
          presupuestos = VPresupuestoDetallado.agrupa_oe_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
          gastos = VGasto.agrupa_oe_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], filtro_partida)
        end


      when ["partida_proyecto", "implementador"] then
         agente = params[:agente]
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.agrupa_partida_proyecto_de_implementador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])

      when ["partida_proyecto", "financiador"] then
        # Si queremos el resumen agrupado...
        if params[:agente] == "todos_agrupados"
          presupuestos = []
          gastos = VGasto.partida_proyecto_financiador_x_grupos( proyecto.id, params[:moneda], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])
        # Para resumen por agente...
        else  
          # Para los agentes del sistema, seleccionamos la agrupacion...
          agt = Agente.find_by_id params[:agente]
          if agt && agt.sistema
            if agt.publico
              agente = agt.local ? @proyecto.financiador_local_publico : @proyecto.financiador_externo_publico
            else
              agente = agt.local ? @proyecto.financiador_local_privado : @proyecto.financiador_externo_privado
            end
          end
          agente ||= params[:agente]
          # Y calculamos presupuesto y gasto
          presupuestos = VPresupuestoDetallado.partida_proyecto_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
          gastos = VGasto.agrupa_partida_proyecto_de_financiador( proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio])
        end
    end 

    datos = presupuestos + gastos 
    titulo =[ _("Resumen de gasto") + " " + proyecto.nombre + ".",
              _("Moneda") + ": " + ( params[:moneda] == 'todas' ?  _('Todas las monedas') : Moneda.find(params[:moneda]).nombre),
              _("Tasa Cambio") + ": " + ( (params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? _('Aplicada') : _('No aplicada') ) + " (" +
                _("importes en %{moneda}") % {:moneda => Moneda.find_by_id( (params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? @proyecto.moneda_id : params[:moneda]).abreviatura } + ").",
              _("Fechas") + ": " + @fecha_de_inicio.to_time.to_s + ' - ' + @fecha_de_fin.to_time.to_s,
              _("Filas") + ": " + (@proyecto.convenio_accion == params[:partida_actividad] ? _("Acción") : params[:partida_actividad].humanize),
	      _("Agente") + ": " + (params[:agente] == "todos" || params[:agente] == "todos_agrupados" ? _("Todos los agentes") : Agente.find(params[:agente]).nombre),
              _("Rol del agente") + ": " + params[:financiador_implementador]  ]

    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_resta => true, :fila_suma => true, :columna_desviacion => true, :clases => ["","2","1_2_td"] } }] unless params[:agente] == "todos_agrupados"
    @resumen = [ :tabla => {:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_resta => true, :fila_suma => true, :columna_desviacion => true, :clases => ["","2","1_2_td"] } } ] unless params[:agente] == "todos_agrupados"

    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_suma => true, :fila_suma => true, :ancho_fila => (9 * 105 + 200),:clases => ["","1","1_2_td"] } }] if params[:agente] == "todos_agrupados"
    @resumen = [ :tabla => {:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => { :columna_suma => true, :fila_suma => true, :ancho_fila => (9 * 105 + 200), :clases => ["","1","1_2_td"] } } ] if params[:agente] == "todos_agrupados"


    nombre_subpartidas = _("Subpartidas por partida")

    # Buscamos las subpartidas para segunda tabla si estamos en un resumen por partidas o por partidas de proyecto.
    @titulo, @titulo[0] = titulo[0..3], nombre_subpartidas
    @subpartidas = [{:cabecera => [ [_("Partida/Subpartida"),"2"], [_("Presupuesto"), "1_2_td"], [_("Gasto"), "1_2_td"], [_("Diferencia"), "1_2_td"] ] } ]
    if params[:partida_actividad] == "partida"
      for partida in Partida.all
        presupuesto = presupuestos.detect{|v| v["fila_id"] == partida.id }
        importe_presupuesto = presupuesto ? presupuesto["importe"] : 0
        gasto = gastos.detect{|v| v["fila_id"] == partida.id }
        importe_gasto = gasto ? gasto["importe"] : 0
        @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [importe_presupuesto ,"1_2_td"], [importe_gasto ,"1_2_td"],
                                          [((importe_presupuesto || 0)  - (importe_gasto || 0 ))  ,"1_2_td"]])
        for subpartida in Subpartida.find_all_by_proyecto_id_and_partida_id( @proyecto.id, partida.id)
          importe_presupuesto = VPresupuestoDetallado.subpartida_partida(proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          importe_gasto = VGasto.subpartida_partida(proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : _("Sin subpartida")), importe_presupuesto, importe_gasto, ((importe_presupuesto || 0)  - (importe_gasto || 0 ))])  if importe_presupuesto or importe_gasto
        end
      end
    elsif  params[:partida_actividad] == "partida_proyecto"
      for partida_proyecto in proyecto.partida_financiacion
        presupuesto = presupuestos.detect{|v| v["fila_id"] == partida_proyecto.id } 
        importe_presupuesto = presupuesto ? presupuesto["importe"] : 0
        gasto = gastos.detect{|v| v["fila_id"] == partida_proyecto.id }
        importe_gasto = gasto ? gasto["importe"] : 0
        @subpartidas.push(:cabecera => [[ partida_proyecto.codigo_nombre , "2" ], [importe_presupuesto ,"1_2_td"], [importe_gasto ,"1_2_td"] , 
                                          [((importe_presupuesto || 0)  - (importe_gasto || 0 ))  ,"1_2_td"]])
        for subpartida in partida_proyecto.subpartida_proyecto proyecto.id
          importe_presupuesto = VPresupuestoDetallado.subpartida_partida_proyecto(proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          importe_gasto = VGasto.subpartida_partida_proyecto(proyecto.id, params[:moneda], agente, @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : _("Sin subpartida")), importe_presupuesto, importe_gasto, ((importe_presupuesto || 0)  - (importe_gasto || 0 ))]) if importe_presupuesto or importe_gasto
        end
      end
    end

    @resumen.push(:listado => {:nombre => nombre_subpartidas, :titulo => @titulo, :lineas => @subpartidas} ) if @subpartidas
  end




	# en proyectos: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| @usuario_identificado.libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = [[_("Todas"), "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]} unless @proyecto.convenio?
    @listado_pac = [[_("Todos"), "todos"]] + @proyecto.pacs.collect{|p| [p.nombre, p.id]} if @proyecto.convenio?
    @listado_agentes = @proyecto.implementador.collect{|e| [e.nombre, e.id]}
    @listado_monedas = @proyecto.moneda.collect{|e| [e.nombre, e.id]}
    # Si hemos especificado un libro para ver el informe
    if params[:selector] && (params[:selector][:tipo] == "agente" || params[:selector][:tipo] == "cuenta")
      @libro = Libro.find_by_id(params[:selector][:libro]) if params[:selector][:tipo] == "cuenta"
      @agente = @proyecto.implementador.find_by_id(params[:selector][:agente]) if params[:selector][:tipo] == "agente"
      @moneda = Moneda.find_by_id(params[:selector][:moneda]) if params[:selector][:tipo] == "agente"
      @moneda = @libro.moneda if @libro 

      # Proyectos de los que obtener informacion
      if @proyecto.convenio?
        @pac = Proyecto.find_by_id(params[:selector][:pac])
        proyectos = @pac ? [ @pac ] : @proyecto.pacs
      else
        @etapa = Etapa.find_by_id(params[:selector][:etapa])
        proyectos = [ @proyecto ]
      end

      fecha_inicio = @etapa ? @etapa.fecha_inicio : nil
      fecha_fin = @etapa ? @etapa.fecha_fin : nil
      # Si es arqueo de un solo libro, lo invocamos tal cual
      if @libro
        arqueo = @libro.arqueo( proyectos, fecha_inicio, fecha_fin )
      # Si es arqueo de un agente, recorremos todos sus libros en la moneda indicada
      elsif @agente && @moneda
        arqueo = { :filas => [], :totales => {}, :entrante => 0, :saliente => 0 }
        @agente.libro.all(:conditions => {:moneda_id => @moneda.id}).each do |libro|
          arqueo_libro = libro.arqueo( proyectos, fecha_inicio, fecha_fin )
          arqueo[:filas] += arqueo_libro[:filas]
          #arqueo[:totales] = arqueo_libro[:totales]
          arqueo[:entrante] += arqueo_libro[:entrante]
          arqueo[:saliente] += arqueo_libro[:saliente]
          # Sumamos los totales por conceptos
          arqueo_libro[:totales].each do |tipo,parcial|
            arqueo[:totales][tipo] ||= {}
            parcial.each do |dir,valor|
              arqueo[:totales][tipo][dir] ||= 0
              arqueo[:totales][tipo][dir] += valor
            end
          end
        end
        # Ordenamos por fecha las filas
        arqueo[:filas].sort! { |x,y| x[:contenido][0] <=> y[:contenido][0] }
      end

      # Incluye los saldos linea a linea
      saldo = 0.0
      arqueo[:filas].each do |fila|
        saldo += fila[:contenido][3].to_f - fila[:contenido][4].to_f
        fila[:contenido].push saldo
      end

      # Prepara la salida de datos
      lineas = Array.new
      lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"], [_("Saldo"),"1_2_td"] ])
      lineas += arqueo[:filas]
      lineas.push( :contenido => [ '','','','','' ] )
      lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [arqueo[:entrante],"1_2_td"], [arqueo[:saliente],"1_2_td"], [arqueo[:entrante] - arqueo[:saliente],"1_2_td"] ])
      arqueo[:totales].each do |k,v|
        lineas.push( :contenido => [ '',_(k),'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      @resumen = Array.new
      nombre = "arqueo_caja"
      titulo = @proyecto.nombre + " / "
      titulo += _("Arqueo ") + (@libro ? _("Cuenta/Caja") + " " + @libro.nombre : _("Cuentas/Cajas de") + " " + @agente.nombre ) + " (" + @moneda.nombre + ") / " 
      if @proyecto.convenio?
        titulo += _("PAC") + ": " + (@pac ? @pac.nombre : _("Todas") )
      else
	titulo += _("Etapa") + ": " + (@etapa ? @etapa.nombre + " " + _("(desde %{fecha_inicio} hasta %{fecha_fin})") % {:fecha_inicio => @etapa.fecha_inicio.strftime('%d/%m/%Y'), :fecha_fin => @etapa.fecha_fin.strftime('%d/%m/%Y')} : _("Todas") )
      end
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

      respond_to do |format|
        format.html do
          render :template => 'comunes/arqueo_caja', :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          nom_fich = "arqueo_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end
    else
      render :template => "comunes/arqueo_caja", :layout => (params[:sin_layout] ? false : true)
    end
  end

	# en proyectos, vista resumen de transferencias (segun la hoja X.A.1 del informe AECID)
  def transferencia
    @listado_etapa = [[_("Todas"), "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]} unless @proyecto.convenio?
    @listado_pac = [[_("Todos"), "todos"]] + @proyecto.pacs.collect{|p| [p.nombre, p.id]} if @proyecto.convenio?


      @etapa = params[:selector] && params[:selector][:etapa] ? @proyecto.etapa.find_by_id(params[:selector][:etapa]) : nil
      @pac = params[:selector] && params[:selector][:pac] ? @proyecto.pacs.find_by_id(params[:selector][:pac]) : nil

      lineas = Array.new

      # Estas van a ser condiciones generales
      moneda_principal = @proyecto.moneda_principal
      moneda_intermedia = @proyecto.moneda_intermedia
      condiciones_base = Hash.new
      if @proyecto.convenio?
        condiciones_base[:proyecto_id] = @pac ? [@pac.id] : @proyecto.pacs.collect{|p| p.id}
        condiciones_base[:remanente] = false unless @pac 
      else
        condiciones_base[:proyecto_id] = [@proyecto.id]
        condiciones_base[:fecha_recibido] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
      end

      # Subvenciones Recibidas (de todos los financiadores?)
      condiciones = condiciones_base.clone
      condiciones["libro.moneda_id"] = moneda_principal.id
      condiciones[:tipo] = "subvencion"
      transferencias_principal = Transferencia.all(:order => "fecha_recibido", :include => ["libro_destino"], :conditions => condiciones)
      tot_recibidos = 0
      tot_gastos = 0 
      lineas_tmp = Array.new
      transferencias_principal.each do |t|
        gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
        tot_gastos += gasto_transferencia 
        # Desglosa las subvenciones segun el financiador (ajustando los porcentajes de gasto)
        t.transferencia_x_agente.each do |txf|
          porcentaje = txf.importe / t.importe_cambiado
          lineas_tmp.push( :contenido => [t.fecha_recibido, txf.importe, gasto_transferencia * porcentaje, "", txf.agente.nombre, t.libro_destino.nombre])
          tot_recibidos += txf.importe
        end
        # Si no hay financiadores definidos, lo pone tambien
        if t.transferencia_x_agente.empty?
          lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_cambiado, gasto_transferencia, "", "", t.libro_destino.nombre])
          tot_recibidos += t.importe_cambiado
        end
      end
      unless lineas_tmp.empty?
        lineas.push( :cabecera => [["0", "1_3"], [_("Subvenciones Recibidas"), "4"] ] )
        lineas.push( :cabecera => [	[_("Fecha"), "1_3"],    [_("%{moneda} recibidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
					[_("Gastos"), "1_2_td"], ["", "1_4"], [_("Financiador"), "1"], [_("Cuenta"), "1"]      ])
        lineas += lineas_tmp
        lineas.push( :contenido => [_("TOTALES"), tot_recibidos, tot_gastos] )
        lineas.push( :contenido => [] )
      end

      # A partir de este punto, solo se tienen en cuenta movimientos sobre el financiador principal
      #condiciones_base["transferencia_x_agente.agente_id"] = @proyecto.agente.id

      # Transferencias a la moneda principal para cuentas sin pais dentro de los del proyecto
      condiciones = condiciones_base.clone
      condiciones["libro.moneda_id"] = moneda_principal.id
      condiciones["libro_destinos_transferencia.moneda_id"] = moneda_principal.id
      condiciones["transferencia_x_agente.agente_id"] = @proyecto.agente.id
      transferencias_principal = Transferencia.all(:order => "fecha_recibido", :include => ["transferencia_x_agente", "libro_origen", "libro_destino"], :conditions => condiciones)
      lineas_sp = Array.new
      lineas_tmp = Array.new
      tot_sp_transferidos = 0
      tot_sp_recibidos = 0
      tot_sp_gastos = 0
      tot_recibidos = 0

      transferencias_principal.each do |t|
        # Coge por un lado los remanentes
        if t.tipo == "remanente" && @proyecto.libro_id == t.libro_destino_id
          lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_cambiado, "", t.nombres_financiadores, t.libro_destino.nombre])
          tot_recibidos += t.importe_cambiado
        # Y por otro el resto de transferencias a la moneda principal que no estuvieran incluidas
        elsif @proyecto.pais.find_by_id(t.libro_destino.pais_id).nil?
          gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
          lineas_sp.push( :contenido => [t.fecha_recibido, t.importe_enviado, t.importe_cambiado, gasto_transferencia, "", (t.tipo == "remanente" ? "(R) " : "") + t.libro_destino.nombre, t.libro_origen.nombre])
          tot_sp_transferidos += t.importe_enviado if t.importe_enviado 
          tot_sp_recibidos += t.importe_cambiado
          tot_sp_gastos += gasto_transferencia
        end
      end
      # Pinta los remanentes
      unless lineas_tmp.empty?
        lineas.push( :cabecera => [["0", "1_3"], [_("Remanentes"), "4"] ] )
        lineas.push( :cabecera => [	[_("Fecha"), "1_3"],
                                        [_("%{moneda} recibidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
					["", "1_4"], [_("Financiador"), "1"], [_("Cuenta"), "1"]      ])
        lineas += lineas_tmp
        lineas.push( :contenido => [_("TOTALES"), tot_recibidos] )
        lineas.push( :contenido => [] )
      end
      # Y despues de los paises, las que no tienen

      # Transferencias a la moneda principal por paises para el financiador principal
      @proyecto.pais.each do |p|
        condiciones = condiciones_base.clone
        condiciones["libro.moneda_id"] = moneda_principal.id
        condiciones["libro_destinos_transferencia.moneda_id"] = moneda_principal.id
        condiciones["libro_destinos_transferencia.pais_id"] = p.id
        transferencias_principal = Transferencia.all(:order => "fecha_recibido", :include => ["libro_origen", "libro_destino"], :conditions => condiciones)

        lineas_tmp = Array.new
        tot_transferidos = 0
        tot_recibidos = 0
        tot_gastos = 0

        transferencias_principal.each do |t|
          # No lo incluye si es un remanente y estamos con consolidado del convenio
          gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
          lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_enviado, t.importe_cambiado, gasto_transferencia, "", (t.tipo == "remanente" ? "(R) " : "") + t.libro_origen.nombre, t.libro_destino.nombre])
          tot_transferidos += t.importe_enviado if t.importe_enviado
          tot_recibidos += t.importe_cambiado
	  tot_gastos += gasto_transferencia
        end
        unless lineas_tmp.empty?
          lineas.push( :cabecera => [["1A", "1_3"], [_("Transferencias %{agente} a cuentas %{moneda} en %{pais}") % {:agente => @proyecto.agente.nombre, :moneda => moneda_principal.abreviatura, :pais => p.nombre}, "4"]] )
          lineas.push( :cabecera => [ [_("Fecha"), "1_3"],
                                    [_("%{moneda} transferidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                    [_("%{moneda} recibidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                    [_("Gastos"), "1_2_td"], ["", "1_4"],
                                    [_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]      ])
          lineas += lineas_tmp
          lineas.push( :contenido => [_("TOTALES"), tot_transferidos, tot_recibidos, tot_gastos] )
          lineas.push( :contenido => [] )
        end
      end

      # Pinta las transferencias a la moneda principal para no pais
      unless lineas_sp.empty?
        lineas.push( :cabecera => [["1A", "1_3"], [_("Transferencias ") + @proyecto.agente.nombre + _(" a otras cuentas ") + moneda_principal.abreviatura, "4"] ] )
        lineas.push( :cabecera => [	[_("Fecha"), "1_3"],
                                        [_("%{moneda} transferidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                        [_("%{moneda} recibidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                        [_("Gastos"), "1_2_td"], ["", "1_4"],
					[_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]      ])
        lineas += lineas_sp
        lineas.push( :contenido => [_("TOTALES"), tot_sp_transferidos, tot_sp_recibidos, tot_sp_gastos] )
        lineas.push( :contenido => [] )
      end

      # Transferencias a la divisa (si la hay) desde la principal por paises y financiadores
      condiciones = condiciones_base.clone
      condiciones["libro.moneda_id"] = moneda_principal.id
      condiciones["libro_destinos_transferencia.moneda_id"] = moneda_intermedia.id if moneda_intermedia

      @proyecto.financiador.each do |f|
        condiciones["transferencia_x_agente.agente_id"] = f.id
        @proyecto.pais.each do |p|
          condiciones["libro_destinos_transferencia.pais_id"] = p.id
          transferencias_divisa = Transferencia.all(:order => "fecha_recibido", :include => ["transferencia_x_agente","libro_origen", "libro_destino"], :conditions => condiciones)

          lineas_tmp = Array.new
          tot_transferidos = 0
          tot_recibidos = 0
          tot_cambiados = 0
          tot_gastos = 0

          # Recoge las transferencias a divisas 
          transferencias_divisa.each do |t|
            gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
            lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_enviado, t.importe_recibido, t.importe_cambiado, t.tasa_cambio, gasto_transferencia, "", (t.tipo == "remanente" ? "(R) " : "") + t.libro_origen.nombre, t.libro_destino.nombre])
            tot_transferidos += t.importe_enviado if t.importe_enviado
            tot_recibidos += t.importe_recibido if t.importe_recibido
            tot_cambiados += t.importe_cambiado if t.importe_cambiado
            tot_gastos += gasto_transferencia
          end
          unless lineas_tmp.empty?
            tc_total = tot_cambiados != 0 ? format("%.5f",tot_recibidos / tot_cambiados) : "" 
            lineas.push( :cabecera => [ ["1B", "1_3"],
                                        [_("Transferencias %{agente} a cuentas %{moneda} en %{pais}") % {:agente => f.nombre, :moneda => moneda_intermedia.abreviatura, :pais => p.nombre}, "4"]
                                      ] )
            lineas.push( :cabecera => [ [_("Fecha"), "1_3"],
                                        [_("%{moneda} enviados") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                        [_("%{moneda} recibidos y cambiados") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                        [_("%{moneda} recibidos") % {:moneda => moneda_principal.abreviatura}, "1_2_td"],
                                        [_("Tasa de Cambio") + " " + moneda_intermedia.abreviatura + "/" + moneda_principal.abreviatura, "1_2_td_g"],
                                        [_("Gastos") + " (" + moneda_intermedia.abreviatura + ")", "1_2_td"],
                                        ["", "1_4"], [_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]     ])
            lineas += lineas_tmp
            lineas.push( :contenido => [_("TOTALES"), tot_transferidos, tot_recibidos, tot_cambiados, tc_total, tot_gastos] )
            lineas.push( :contenido => [] )
          end
        end
      end if @proyecto.moneda_intermedia


      # Transferencias a las monedas locales por países
      condiciones = condiciones_base.clone
      @proyecto.pais.each do |p|
        condiciones["libro.pais_id"] = p.id
        p.moneda.each do |m|
          #condiciones.delete(:tipo)
          condiciones["libro.moneda_id"] = m.id
          transferencias_locales = Transferencia.all(:order => "fecha_recibido", :include => ["libro_destino"], :conditions => condiciones)

          lineas_tmp = Array.new
          tot_transferidos_princ = 0
          tot_transferidos_sec = 0
          tot_recibidos_princ = 0
          tot_recibidos_sec = 0
          tot_cambiados_princ = 0
          tot_cambiados_sec = 0
          tot_gastos = 0

          transferencias_locales.each do |t|
            if	t.libro_origen && (t.libro_origen.moneda_id == moneda_principal.id || (moneda_intermedia && t.libro_origen.moneda_id == moneda_intermedia.id))
              gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }

              # Separamos si existe o no existe moneda intermedia pues el formato final sera distinto
              if moneda_intermedia
                # Si la transferencia es desde moneda principal
                lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_enviado, "", t.importe_recibido, "", t.importe_cambiado, t.tasa_cambio, "", gasto_transferencia, "", t.nombres_financiadores, t.libro_destino.agente.nombre, (t.tipo == "remanente" ? "(R) " : "") + t.libro_origen.nombre, t.libro_destino.nombre]) if t.libro_origen.moneda == moneda_principal
                # Si la transferencia es desde la moneda intermedia
                lineas_tmp.push( :contenido => [t.fecha_recibido, "", t.importe_enviado, "", t.importe_recibido, t.importe_cambiado, "", t.tasa_cambio, gasto_transferencia, "", t.nombres_financiadores, t.libro_destino.agente.nombre, (t.tipo == "remanente" ? "(R) " : "") + t.libro_origen.nombre, t.libro_destino.nombre]) if t.libro_origen.moneda == moneda_intermedia
              else
                # Cuando no existe la moneda intermedia, eliminamos sus campos
                lineas_tmp.push( :contenido => [t.fecha_recibido, t.importe_enviado, t.importe_recibido, t.importe_cambiado, t.tasa_cambio, gasto_transferencia, "", t.nombres_financiadores, t.libro_destino.agente.nombre, (t.tipo == "remanente" ? "(R) " : "") + t.libro_origen.nombre, t.libro_destino.nombre])
              end

              tot_transferidos_princ += t.importe_enviado if t.importe_enviado && t.libro_origen.moneda == moneda_principal
              tot_transferidos_sec += t.importe_enviado if t.importe_enviado && moneda_intermedia && t.libro_origen.moneda == moneda_intermedia
              tot_recibidos_princ += t.importe_recibido if t.importe_recibido && t.libro_origen.moneda == moneda_principal
              tot_recibidos_sec += t.importe_recibido if t.importe_recibido && moneda_intermedia && t.libro_origen.moneda == moneda_intermedia
              tot_cambiados_princ += t.importe_cambiado if t.importe_cambiado && t.libro_origen.moneda == moneda_principal
              tot_cambiados_sec += t.importe_cambiado if t.importe_cambiado && moneda_intermedia && t.libro_origen.moneda == moneda_intermedia 
              tot_gastos += gasto_transferencia
            end
          end
          unless lineas_tmp.empty?
            tc_total_princ = tot_cambiados_princ != 0 ? format("%.5f",tot_recibidos_princ / tot_cambiados_princ) : ""
            tc_total_sec = tot_cambiados_sec != 0 ? format("%.5f",tot_recibidos_sec / tot_cambiados_sec) : ""
            lineas.push( :cabecera => [["2", "1_3"], [p.nombre + " (" + m.nombre + ")", "4"] ] )
            cabecera  = [ [_("Fecha"), "1_3"], [_("%{moneda} enviados") % {:moneda => moneda_principal.abreviatura}, "1_2_td"]]
            cabecera += [ [_("%{moneda} enviados") % {:moneda => moneda_intermedia.abreviatura}, "1_2_td"] ] if moneda_intermedia
            cabecera += [ [_("%{moneda} recibidos y cambiados") % {:moneda => moneda_principal.abreviatura}, "1_2_td"] ]
            cabecera += [ [_("%{moneda} recibidos y cambiados") % {:moneda => moneda_intermedia.abreviatura}, "1_2_td"] ] if moneda_intermedia
            cabecera += [ [_("%{moneda} recibidos") % {:moneda => m.abreviatura}, "1_2_td"] ]
            cabecera += [ [_("Tasa de Cambio") + " " + m.abreviatura + "/" + moneda_principal.abreviatura, "1_2_td_g"] ]
            cabecera += [ [_("Tasa de Cambio") + " " + m.abreviatura + "/" + moneda_intermedia.abreviatura, "1_2_td_g"] ] if moneda_intermedia
            cabecera += [ [_("Gastos") + " (" + m.abreviatura + ")", "1_2_td"],
                          ["", "1_4"], [_("Financiador"), "3_4"], [_("Destinatario"), "3_4"], [_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]     ]
            lineas.push( :cabecera => cabecera )
            lineas += lineas_tmp
            lineas.push( :contenido => [_("TOTALES"), tot_transferidos_princ, tot_transferidos_sec, tot_recibidos_princ, tot_recibidos_sec, tot_cambiados_princ + tot_cambiados_sec, tc_total_princ, tc_total_sec, tot_gastos] ) if moneda_intermedia
            lineas.push( :contenido => [_("TOTALES"), tot_transferidos_princ, tot_recibidos_princ, tot_cambiados_princ, tc_total_princ, tot_gastos] ) unless moneda_intermedia
            lineas.push( :contenido => [] )
          end
        end
      end 

      @resumen = Array.new
      nombre = "transferencias"
      titulo = _("Resumen Transferencias ") + @proyecto.nombre + " / "
      if @proyecto.convenio?
        titulo += _("PAC") + ": " + (@pac ? @pac.nombre : "Todos")
      else
        titulo += _("Etapa") + ": " + (@etapa ? @etapa.nombre + " " + _("(desde %{fecha_inicio} hasta %{fecha_fin})") % {:fecha_inicio => @etapa.fecha_inicio.strftime('%d/%m/%Y'), :fecha_fin => @etapa.fecha_fin.strftime('%d/%m/%Y')} : _("Todas") )
      end
      resumen_error = @proyecto.pais.size == 0 ? 
	_("No existe ningún país definido para el proyecto.<br/>Configure los paises de actuación en las Relaciones del Menú de Configuración del proyecto.") : nil
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas, :resumen_error => resumen_error})


      respond_to do |format|
        format.html do
          render :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          nom_fich = "resumen_transferencias_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end
  end

	# En proyectos, vista resumen de estado de tesoreria (segun la hoja X.A.2 del informe AECID)
  def estado_tesoreria
    @listado_etapa = [[_("Todas"), "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]} unless @proyecto.convenio?
    @listado_pac = @proyecto.pacs.collect{|e| [e.nombre, e.id]} if @proyecto.convenio?
    @listado_agente = @proyecto.financiador.collect{|e| [e.nombre, e.id]}

    unless params[:selector]
      params[:selector] = {:agente => @proyecto.agente.id}
    end

    if @proyecto.convenio?
      @etapa = nil
      proyecto = @proyecto.pacs.find_by_id(params[:selector][:pac]) || @proyecto.pacs.first || @proyecto
    else
      @etapa = @proyecto.etapa.find_by_id(params[:selector][:etapa])
      proyecto = @proyecto
    end
    @agente = @proyecto.financiador.find_by_id(params[:selector][:agente])

    lineas = []
    (cab_col, cab_fila, filas) = proyecto.tesoreria(@agente, @etapa)

    cabecera = [ [_("Ingresos y Transferencias"),"1"] ]
    cab_col.each do |cab|
      p = Pais.find_by_id(cab[:pais_id])
      m = Moneda.find_by_id(cab[:moneda_id])
      cabecera.push( [ m.nombre + ( p ? " " + p.nombre : ""), "1_2_td"] )
    end

    lineas.push( :cabecera => cabecera )
    cab_fila.zip(filas).each do |cab, fila|
      estilo = [ ['',"1"] ]
      estilo += fila.collect{ ['','1_2_td' ] } unless cab == _("Tipo de cambio aplicable")
      estilo += fila.collect{ ['','1_2_td_g' ] }   if cab == _("Tipo de cambio aplicable")
      lineas.push( :estilo => estilo, :contenido => [ cab ] + fila )
    end

    @resumen = Array.new
    nombre = "tesoreria"
    titulo = _("Resumen Tesoreria") + " " + @proyecto.nombre + " / " 
    titulo += _("Etapa") + ": " + (@etapa ? @etapa.nombre + " " + _("(desde %{fecha_inicio} hasta %{fecha_fin})") % {:fecha_inicio => @etapa.fecha_inicio.strftime('%d/%m/%Y'), :fecha_fin => @etapa.fecha_fin.strftime('%d/%m/%Y')} : _("Todas") ) unless @proyecto.convenio?
    titulo += _("PAC") + ": " + proyecto.nombre if @proyecto.convenio?
    @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

    respond_to do |format|
      format.html do
        render :layout => (params[:sin_layout] ? false : true)
      end
      format.xls do
        nom_fich = "resumen_tesoreria_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
      end
    end
  end

	# En proyectos, vista resumen de la matriz por etapa y pais
  def matriz
    @listado_etapa = [[_("Todas"), "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_pais =  [[_("Todos"), "todos"]] + @proyecto.pais.collect{ |p| [p.nombre, p.id] }.sort{ |a, b| a[0] <=> b[0]}

      @pais = params[:selector] && params[:selector][:pais] ? Pais.find_by_id(params[:selector][:pais]) : nil 
      @etapa = params[:selector] && params[:selector][:etapa] ? Etapa.find_by_id(params[:selector][:etapa]) : nil 
      seguimiento = params[:selector] && params[:selector][:tipo] == "seguimiento"
      ocultar_comentarios = params[:selector] && params[:selector][:ocultar_comentarios] == "1"
      #@columnas_nombre = [_("descripcion"), _("indicadores"), _("fuentes de verificacion"), _("hipótesis")] unless seguimiento
      #@columnas_nombre = [_("descripcion"), _("indicadores"), _("% realizado"), _("comentarios"), _("fuentes de verificacion"), _("hipótesis")] if seguimiento
      lineas = Array.new
      condiciones = { "proyecto_id" =>  @proyecto.id}
      condiciones["actividad_x_pais.pais_id"] = @pais.id if @pais 
      condiciones["actividad_x_etapa.etapa_id"] = @etapa.id if @etapa 
      filas_objetivos = Array.new
      filas_resultados = Array.new
      filas_actividades = Array.new 

      nivel_acciones = @proyecto.convenio ? @proyecto.convenio.convenio_accion : @proyecto.convenio_accion

      # Recorre desde objetivos especificos hacia abajo
      @proyecto.objetivo_especifico.all.each do |objetivo|
        existen_actividades = false
        cabecera_actividades = true if nivel_acciones == "objetivo_especifico"
        objetivo.resultado.all.each do |resultado|
          existen_actividades = false
          cabecera_actividades = true if nivel_acciones.nil? || nivel_acciones == "resultado"
          actividades = resultado.actividad.all(:order => "codigo", :include => [:actividad_x_pais, :actividad_x_etapa], :conditions => condiciones )
          actividades_resultado = []
          actividades.each do |actividad|
            nuevas_actividades = desglosa_actividades(actividad,@etapa,seguimiento,ocultar_comentarios)
            actividades_resultado += nuevas_actividades 
            existen_actividades = true if nuevas_actividades.length > 0 
          end
          if cabecera_actividades && actividades_resultado.length > 0
            objeto_padre = _("Resultado") + " " + resultado.codigo if nivel_acciones.nil?
            objeto_padre = _("Acción") + " " + (nivel_acciones == "resultado" ? resultado.codigo : objetivo.codigo) unless nivel_acciones.nil?
            filas_actividades.push(:cabecera => [ [_("Actividades") + " " + objeto_padre,"3_2"], [_("Recursos"),"1_td"], [_("Costes"),"1_td"] ]) unless seguimiento
            filas_actividades.push(:cabecera => [ [_("Actividades") + " " + objeto_padre,"3_2"], ["% " + _("Actividad"), "3_4"], [_("Recursos"),"1_td"], [_("Costes"),"1_2_td"], [_("Gastos"), "1_2_td"] ]) if seguimiento
            cabecera_actividades = false
          end
          filas_actividades += actividades_resultado if actividades_resultado.length > 0 
          filas_resultados += desglosa_fuentes(resultado, seguimiento, ocultar_comentarios) if existen_actividades
        end
        filas_objetivos += desglosa_fuentes(objetivo, seguimiento, ocultar_comentarios) if existen_actividades 
      end

      # Y le incluye las actividades globales (sin resultado asociado)
      condiciones[:resultado_id] = nil
      filas_actividades_comunes = [] 
      @proyecto.actividad.all(:order => "codigo", :include => [:actividad_x_pais, :actividad_x_etapa], :conditions => condiciones).each do |actividad|
        filas_actividades_comunes += desglosa_actividades(actividad,@etapa,seguimiento,ocultar_comentarios)
      end

      # Va rellenando las lineas
      #  primero el objetivo general
      if (@proyecto.objetivo_general && @proyecto.objetivo_general.descripcion != "")
        lineas.push(:cabecera => [ [_("Objetivo General"), "4"] ])
        lineas.push(:contenido => [ @proyecto.objetivo_general.descripcion ])
      end
      #  y continua con el objetivo especifico
      lineas.push(:cabecera => [ [(nivel_acciones == "objetivo_especifico" ? _("Acciones") : _("Descripción")),"3_2"], [_("Indicadores"),"1"], [_("Fuentes de verificación"),"1"], [_("Hipótesis"),"1"] ]) unless seguimiento
      lineas.push(:cabecera => [ [(nivel_acciones == "objetivo_especifico" ? _("Acciones") : _("Descripción")),"3_2"], [_("Indicadores"),"1"], ["% " + _("Indicador"),"1"], [_("Fuentes de verificación"),"1"] ]) if seguimiento
      lineas += filas_objetivos
      lineas.push(:cabecera => [ [(nivel_acciones == "resultado" ? _("Acciones") : _("Resultados")),"3_2"], ["","1"], ["","1"], ["","1"] ]) unless seguimiento
      lineas.push(:cabecera => [ [(nivel_acciones == "resultado" ? _("Acciones") : _("Resultados")),"3_2"], ["","1"], ["","1"], ["","1"] ]) if seguimiento 
      lineas += filas_resultados 
      lineas += filas_actividades
      if filas_actividades_comunes.length > 0
        lineas.push(:cabecera => [ [_("Actividades Globales"),"3_2"], [_("Recursos"),"1_td"], [_("Costes"),"1_td"] ]) unless seguimiento
        lineas.push(:cabecera => [ [_("Actividades Globales"),"3_2"], ["% " + _("Actividad"), "3_4"], [_("Recursos"),"1_td"], [_("Costes"),"1_2_td"], [_("Gastos"), "1_2_td"] ]) if seguimiento
        lineas += filas_actividades_comunes
      end

      @resumen = Array.new
      nombre = "matriz"
      titulo  = ( params[:selector] && params[:selector][:tipo] == "seguimiento" ? _("Matriz de Seguimiento") : _("Matriz de Formulación") ) + " "
      titulo += @proyecto.nombre + " / " + _("País") + ": " + (@pais ? @pais.nombre : _("Todos")) + " / "
      titulo += _("Etapa") + ": " + (@etapa ? @etapa.nombre + " " + _("(desde %{fecha_inicio} hasta %{fecha_fin})") % {:fecha_inicio => @etapa.fecha_inicio.strftime('%d/%m/%Y'), :fecha_fin => @etapa.fecha_fin.strftime('%d/%m/%Y')} : _("Todas") ) 
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

      respond_to do |format|
        format.html do
          render :action => "matriz", :layout => (params[:sin_layout] ? false : true)
        end
        format.xls do
          nom_fich = "resumen_matriz_" + @proyecto.nombre.gsub(' ','_') + "_" + Time.now.strftime("%Y-%m-%d")
          render 'comunes_xls/resumen', :xls => nom_fich, :layout => false
        end
      end

  end

        # Para un objetivo especifico o un resultado, saca hipotesis, indicadores y fuentes de verificacion
  def desglosa_fuentes(objeto, seguimiento=false, ocultar_comentarios=false)
    filas = Array.new
    titulo_objeto = objeto.codigo + ": " + objeto.descripcion
    comentarios = objeto.comentario
    titulo_objeto += "\n\n* ".html_safe + comentarios.collect{|c| c.texto}.join("\n* ".html_safe) unless !seguimiento || ocultar_comentarios || comentarios.empty?
    hipotesis = objeto.hipotesis
    indicadores = objeto.indicador
    objeto_repetido = nil
    # Si no hay indicadores al menos muestra el codigo del objeto
    filas.push( :contenido => [ titulo_objeto, "", "", "" ] ) if indicadores.length == 0
    # Si hay indicadores se los recorre para rellenar las filas
    indicadores.each do |indicador|
      indicador_repetido = nil
      titulo_indicador = indicador.codigo + ": " + indicador.descripcion
      comentarios = indicador.comentario
      #titulo_indicador += "\n\n* ".html_safe + comentarios.collect{|c| c.texto}.join("\n* ".html_safe) unless !seguimiento || ocultar_comentarios || comentarios.empty?
      fuentes = indicador.fuente_verificacion
      # Si no hay fuentes de verificacion al menos pone el indicador...
      # ... para formulacion
      filas.push( :contenido => [       objeto_repetido ? "" : titulo_objeto,
                                        titulo_indicador,
                                        "",
                                        (hipotesis.length>0 ? hipotesis.shift.descripcion : "") ]) if fuentes.length == 0 && !seguimiento
      # ... y para seguimiento
      filas.push( :contenido => [       objeto_repetido ? "" : titulo_objeto,
                                        titulo_indicador,
                                        (indicador.porcentaje_actual * 100).to_i.to_s + "%" + (indicador.comentario_porcentaje_actual != "" && !ocultar_comentarios ? " - " + indicador.comentario_porcentaje_actual : ""),
                                        "" ]) if fuentes.length == 0 && seguimiento
      # Si hay fuentes de verificacion las recorre para rellenar las filas
      fuentes.each do |fuente|
        # ... para formulacion
        filas.push( :contenido => [     objeto_repetido ? "" : titulo_objeto,
                                        indicador_repetido ? "" : titulo_indicador,
                                        fuente.codigo + ": " + fuente.descripcion,
                                        (hipotesis.length>0 ? hipotesis.shift.descripcion : "") ]) unless seguimiento
        # ... y para seguimiento
        filas.push( :contenido => [     objeto_repetido ? "" : titulo_objeto,
                                        indicador_repetido ? "" : titulo_indicador,
                                        indicador_repetido ? "" : (indicador.porcentaje_actual * 100).to_i.to_s + "%" + (indicador.comentario_porcentaje_actual != "" && !ocultar_comentarios ? " - " + indicador.comentario_porcentaje_actual : ""),
                                        (fuente.completada ? "(" + _("Completada") + ") " : "") +
                                          fuente.codigo + ": " + fuente.descripcion ]) if seguimiento
        objeto_repetido = true
        indicador_repetido = true
      end
      objeto_repetido = true
    end
    # Si queda alguna hipotesis la mete
    hipotesis.each do |hip|
      filas.push( :contenido => [       "", "", "", hip.descripcion ])
    end unless seguimiento
    return filas
  end

        # Para una actividad, saca partidas de financiador e importes totales
  def desglosa_actividades(objeto,etapa, seguimiento=false, ocultar_comentarios=false)
    hash = Hash.new 
    filas = Array.new
    importe_total = 0
    gasto_total = 0
    # Se recorre todos los presupuestos por actividad
    objeto.presupuesto_x_actividad.each do |pxa|
      # Y en el caso de que el presupuesto sea de la etapa relacionada y tenga asignada una partida lo mete
      if pxa.presupuesto && pxa.presupuesto.partida && (etapa.nil? || pxa.presupuesto.etapa_id == etapa.id)
        partida_asociada=pxa.presupuesto.partida.partida_asociada(@proyecto.id) if pxa.presupuesto && pxa.presupuesto.partida
        if partida_asociada
          # aplica la tasa de cambio
          importe = pxa.importe * pxa.presupuesto.tasa_cambio
          hash[partida_asociada.nombre] = Hash.new unless hash[partida_asociada.nombre]
          hash[partida_asociada.nombre][:presupuesto] += importe if hash[partida_asociada.nombre][:presupuesto]
          hash[partida_asociada.nombre][:presupuesto] = importe unless hash[partida_asociada.nombre][:presupuesto]
          importe_total += importe
        end
      end
    end
    # Se recorre todos los gastos por actividad
    objeto.gasto_x_actividad.each do |gxa|
      # Y en el caso de que el presupuesto sea de la etapa relacionada y tenga asignada una partida lo mete
      if gxa.gasto && gxa.gasto.partida && (etapa.nil? || (gxa.gasto.fecha >= etapa.fecha_inicio && gxa.gasto.fecha <= etapa.fecha_fin))
        partida_asociada = gxa.gasto.partida.partida_asociada(@proyecto.id)
        if partida_asociada
          # obtiene y aplica la tasa de cambio
          tc = gxa.gasto.tasa_cambio_proyecto(@proyecto.id)
          if tc
            importe = gxa.importe * tc.tasa_cambio
            hash[partida_asociada.nombre] = Hash.new unless hash[partida_asociada.nombre]
            hash[partida_asociada.nombre][:gasto] += importe if hash[partida_asociada.nombre][:gasto]
            hash[partida_asociada.nombre][:gasto] = importe unless hash[partida_asociada.nombre][:gasto]
            gasto_total += importe
          end
        end
      end
    end if seguimiento
    titulo_objeto = objeto.codigo + ": " + objeto.descripcion
    estado_actual = objeto.estado_actual(etapa) if seguimiento
    comentario_actividad = (estado_actual ? (estado_actual.porcentaje * 100).to_i.to_s + "%" + ((estado_actual.comentario && estado_actual.comentario != "" && !ocultar_comentarios ) ? " - " + estado_actual.comentario : "") : "") if seguimiento 
    # Si no hay presupuesto para la actividad, mete al menos la actividad
    filas.push( :contenido => [         titulo_objeto, "", ""]) if hash.length == 0 && !seguimiento
    filas.push( :contenido => [         titulo_objeto, comentario_actividad, "", ""]) if hash.length == 0 && seguimiento
    # Si hay datos en el hash anterior, va rellenando las lineas
    if importe_total != 0 || gasto_total != 0 
      filas.push( :contenido => [       titulo_objeto, _("Total"), importe_total]) unless seguimiento
      filas.push( :contenido => [       titulo_objeto, comentario_actividad, _("Total"), importe_total, gasto_total]) if seguimiento
    end
    hash.each do |k,v|
      filas.push( :contenido => [       "", k, v[:presupuesto] ]) unless seguimiento
      filas.push( :contenido => [       "", "", k, v[:presupuesto], v[:gasto] ]) if seguimiento
    end
    return filas
  end


#+++
# Resumen de Actividades
#---

	# en proyectos: genera el informe de seguimiento tecnico 
  def seguimiento_tecnico 
    @listado_pac = [[_("Todas"),nil]] + @proyecto.pacs.collect{|e| [e.nombre, e.id]} if @proyecto.convenio?
    @fecha_de_fin = DateTime.now.to_date unless params["fecha_fin"]
    @fecha_de_fin = Date.strptime(params["fecha_fin"]["(3i)"] + '-' + params["fecha_fin"]["(2i)"] + '-' + params["fecha_fin"]["(1i)"], '%d-%m-%Y') if params["fecha_fin"]

    if params[:selector] && params[:selector][:pdf] == "1"
      begin
        url = url_for(:only_path => false, :action => :seguimiento_tecnico_render, :id => 33, :fecha_de_fin => @fecha_de_fin.to_time.to_i.to_s)
        kit = Shrimp::Phantom.new( url, {}, {"_session_id" => cookies[:_session_id]})
        send_file(kit.to_pdf, :filename => @proyecto.nombre + "_" + _("seguimiento_tecnico") + '.pdf', :type => 'application/pdf', :disposition => 'inline')
      rescue => ex
        logger.error ex.message
        logger.error ex.backtrace
        msg_error _("Se produjo un error en el módulo de exportación a PDF.")
        seguimiento_tecnico_render
      end
    else
      seguimiento_tecnico_render
    end
  end
	# renderiza el informe de seguimiento tecnico (es necesario hacerlo asi para poder generar pdfs)
  def seguimiento_tecnico_render
    @fecha_de_fin ||= Time.at(params[:fecha_de_fin].to_i).to_date if params[:fecha_de_fin]
    params[:pac] = params[:id] if params[:id]
    params[:pac] ||= @proyecto.pacs.first.id if @proyecto.convenio?

    if @proyecto.convenio?
      pac = Proyecto.find_by_id(params[:pac])
      @proyectos = pac || @proyecto.pacs
    else
      @proyectos = [@proyecto] unless @proyecto.convenio?
    end
    unless @fecha_de_fin.nil?
      estado_seguimiento_tecnico
      @activs = @proyecto.actividad.sort_by(&:codigo).group_by(&:resultado)
    end
    render :action => "seguimiento_tecnico", :layout => (params[:fecha_de_fin] ? false : true)
  end

  def estado_seguimiento_tecnico
    # Seleccionamos actividades y resultados y objetivos activos en etapas dentro de las fechas
    #@actividades = @proyecto.actividad.all(:include => ["etapa"], :conditions => ["etapa.fecha_inicio <= ?", @fecha_de_fin])
    #@resultados = @proyecto.resultado.all(:include => ["actividad"], :conditions => {"actividad.id" => @actividades})
    #@oe = @proyecto.objetivo_especifico.all(:include => ["resultado"], :conditions => {"resultado.id" => @resultados})
    @actividades = Actividad.all(:order => "actividad.codigo", :include => ["etapa"], :conditions => ["etapa.fecha_inicio <= ? AND actividad.proyecto_id IN (?)", @fecha_de_fin, @proyectos])
    @resultados = Resultado.all(:order => "resultado.codigo", :include => ["actividad"], :conditions => {"actividad.id" => @actividades, "resultado.proyecto_id" => @proyectos})
    @oe = ObjetivoEspecifico.all(:order => "objetivo_especifico.codigo", :include => ["resultado"], :conditions => {"resultado.id" => @resultados, "objetivo_especifico.proyecto_id" => @proyectos})
  end

end

